<!DOCTYPE html>

<html>
    <body>
        {% for upload in filedata %}
        <h3>{{ upload.data.filename }}</h3>
        {% endfor %}
        <form method="POST" enctype="multipart/form-data">
            {{ form.errors }}
            {{ form.hidden_tag() }}
            {% for upload in form.uploads.entries %}
            <p>
            {{ upload }}
            </p>
            {% endfor %}
            <p>
                <input type="submit" value="Submit">
            </p>
        </form>
    </body>
</html>
